#include "../../BarinkEngine/src/BarinkEngine.h"
#include "../../BarinkEngine/src/Scene/Components.h"
#include "../../BarinkEngine/src/Scene/Scene.h"
#include "../../BarinkEngine/src/Scene/Entity.h"
#include "../../BarinkEngine/src/AssetManager/ModelImporter.h"
#include <imgui.h>
#include "GUI.h"
#include "Util.h"
#include <entt/entt.hpp>

/*
* Define globals
*/
Scene scene;
VertexArray va = VertexArray();
GpuBuffer vertexBuffer = GpuBuffer();
GpuBuffer elementBuffer = GpuBuffer();
BarinkEngine::Renderable* renderable;
BarinkEngine::SceneObject* object;
Entity cube;
/*
* Runs once at startup 
* - USe to initialize the game/sandbox/demo
*/
void Start() {
   
    cube  = scene.AddEntity((std::string&)"cube");
    auto& render3DComponent = cube.AddComponent<BarinkEngine::Render3DComponent>();
    auto importer = BarinkEngine::ModelImporter();

    va.Create();
    va.Bind();

    object = importer.Import("build/Debug/Models/Cube.obj");
    renderable = object->renderable;
    render3DComponent.ElementCount = static_cast<unsigned int>(renderable->mesh->elements.size());
    
    vertexBuffer.createBuffer();
    vertexBuffer.Bind(false);
    vertexBuffer.setBufferData((void*)&renderable->mesh->vertices[0], renderable->mesh->vertices.size() * sizeof(BarinkEngine::Vertex), false);

    elementBuffer.createBuffer();
    elementBuffer.Bind(true);
    elementBuffer.setBufferData((void*)&renderable->mesh->elements[0], renderable->mesh->elements.size() * sizeof(unsigned int), true);

    va.AttachAttribute(0, 3, sizeof(BarinkEngine::Vertex));

    glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, sizeof(BarinkEngine::Vertex), (void*)0);
    glEnableVertexAttribArray(0);
    glEnableVertexAttribArray(1);

    va.Unbind();
    vertexBuffer.Unbind(false);
    elementBuffer.Unbind(true);

    render3DComponent.VAO = va.getID();
    render3DComponent.IBO = elementBuffer.getBufferID();
    render3DComponent.color = glm::vec3(1.0f, 0.0f, 0.0f);
    std::cout << render3DComponent.ElementCount << std::endl;
}

/*
* Runs every frame
* - Use to draw Immediate mode graphics (Not meant for HUD's )
*/
void ImmediateGraphicsDraw() 
{
    // Show internal BarinkEngine stats
    ShowStats();

    ImGui::Begin("Render edit");
        
    auto a = cube.GetComponent<BarinkEngine::Render3DComponent>();
    
    ImGui::DragFloat3("Color", &a->color[0], 0.01f, 0.0f, 1.0f);

    ImGui::End();




}

/*
* Runs every frame
* - Meant for game logic ( non-physics related) 
*/
void Update()
{
    renderer.Render(scene);
   
}

/*
* Runs at the end of the program
* - Meant for cleanup
*/
void Stop()
{
}